import * as React from 'react';
import { View, TextInput, TouchableOpacity, Text } from 'react-native';
import PropTypes from 'prop-types';

import { getColorPalette } from './assets/colors';
import styles from './styles';

type State = {
	focused: boolean,
	passwordVisible: boolean,
}

class Input extends React.Component<Props, State> {
	constructor(props: any) {
		super(props);
		this.state = {
			focused: false,
			passwordVisible: props.secureTextEntry,
		};
	}

	onChangeTextInputState = () => {
		this.setState({ focused: !this.state.focused });
	};

	onTogglePasswordPress = () => {
		this.setState({ passwordVisible: !this.state.passwordVisible });
	};

	render() {
		const {
			theme,
			accentColor,

			disabled,
			secureTextEntry,
			interactive,

			overlayComponent,
			overlayGravity,
			onOverlayPress,
			...remainingProps
		} = this.props;

		const { focused, passwordVisible } = this.state;

		const getComponentOverlay = () => {
			if (!overlayComponent) return;

			const isFunction = typeof overlayComponent === 'function';
			const overlay = () => isFunction ? overlayComponent(focused) : overlayComponent;
			const overlayWithAction = () => (
				<TouchableOpacity style={styles.overlayAction(interactive, overlayGravity)} onPress={onOverlayPress}>
					{overlay()}
				</TouchableOpacity>
			);

			const combinedStyles = [
				styles.overlay(overlayGravity),
				this.props.overlayStyle,
			];
			const overlayProps = interactive ? {} : {};

			return (
				<View style={combinedStyles} {...overlayProps}>
					{onOverlayPress
						? overlayWithAction()
						: overlay()
					}
				</View>
			);
		};

		const getTogglePasswordButton = () => {
			if (!secureTextEntry) return;

			const togglePasswordText =
				passwordVisible ? 'Show' : 'Hide';

			return (
				<TouchableOpacity
					style={styles.togglePasswordButton}
					onPress={this.onTogglePasswordPress}>

					<Text style={styles.togglePasswordButtonText(colorPalette.text(disabled))}>
						{togglePasswordText}
					</Text>
				</TouchableOpacity>
			);
		};

		const colorPalette = getColorPalette(theme, accentColor);

		const combinedStyles = [
			styles.input(colorPalette.text(disabled), overlayComponent && overlayGravity, secureTextEntry),
			this.props.style,
		];

		return (
			<View style={[styles.container(colorPalette.underline(focused)), this.props.containerStyle]}>
				<TextInput
					{...remainingProps}
					style={combinedStyles}
					onFocus={this.onChangeTextInputState}
					onBlur={this.onChangeTextInputState}
					placeholderTextColor={colorPalette.placeholder}
					underlineColorAndroid={colorPalette.underline(focused)}
					keyboardAppearance={theme}
					editable={!disabled}
					selectTextOnFocus={!disabled}
					secureTextEntry={passwordVisible} />

				{getTogglePasswordButton()}
				{getComponentOverlay()}
			</View>
		);
	}
}

Input.propTypes = {
	theme: PropTypes.oneOf(['light', 'dark']),
	accentColor: PropTypes.string,

	containerStyle: PropTypes.any,
	style: PropTypes.any,
	disabled: PropTypes.bool,
	secureTextEntry: PropTypes.bool,
	interactive: PropTypes.bool,

	overlayComponent: PropTypes.func,
	overlayStyle: PropTypes.any,
	overlayGravity: PropTypes.oneOf(['flex-start', 'center', 'flex-end']),
	onOverlayPress: PropTypes.func,
};

Input.defaultProps = {
	theme: require('../config').theme,
	accentColor: require('../config').accent_color,

	disabled: false,
	interactive: true,

	overlayGravity: 'flex-start',
};

export default Input;
